@@ -94,27 +94,17 @@ pub enum Instruction {
         constant: ConstantIndex,
     },
     ValueExport {
-        name: ConstantIndex,
+        name: u8,
         value: u8,
     },
     Import {
         register: u8,
-        constant: ConstantIndex,
-    },
-    MakeTuple {
-        register: u8,
-        start: u8,
-        count: u8,
     },
     MakeTempTuple {
         register: u8,
         start: u8,
         count: u8,
     },
-    MakeList {
-        register: u8,
-        size_hint: usize,
-    },
     MakeMap {
         register: u8,
         size_hint: usize,
@@ -129,6 +119,25 @@ pub enum Instruction {
         count: u8,
         element_register: u8,
     },
+    SequenceStart {
+        register: u8,
+        size_hint: usize,
+    },
+    SequencePush {
+        sequence: u8,
+        value: u8,
+    },
+    SequencePushN {
+        sequence: u8,
+        start: u8,
+        count: u8,
+    },
+    SequenceToList {
+        sequence: u8,
+    },
+    SequenceToTuple {
+        sequence: u8,
+    },
     Range {
         register: u8,
         start: u8,
@@ -158,6 +167,11 @@ pub enum Instruction {
         register: u8,
         iterable: u8,
     },
+    SimpleFunction {
+        register: u8,
+        arg_count: u8,
+        size: usize,
+    },
     Function {
         register: u8,
         arg_count: u8,
@@ -310,29 +324,20 @@ pub enum Instruction {
         register: u8,
         value: u8,
     },
-    ListPushValue {
-        list: u8,
-        value: u8,
-    },
-    ListPushValues {
-        list: u8,
-        values_start: u8,
-        count: u8,
-    },
-    ListUpdate {
-        list: u8,
-        index: u8,
-        value: u8,
-    },
     Index {
         register: u8,
         value: u8,
         index: u8,
     },
+    SetIndex {
+        register: u8,
+        index: u8,
+        value: u8,
+    },
     MapInsert {
         register: u8,
+        key: u8,
         value: u8,
-        key: ConstantIndex,
     },
     MetaInsert {
         register: u8,
@@ -343,21 +348,21 @@ pub enum Instruction {
         register: u8,
         value: u8,
         id: MetaKeyId,
-        name: ConstantIndex,
+        name: u8,
     },
     MetaExport {
-        value: u8,
         id: MetaKeyId,
+        value: u8,
     },
     MetaExportNamed {
-        value: u8,
         id: MetaKeyId,
-        name: ConstantIndex,
+        name: u8,
+        value: u8,
     },
     Access {
         register: u8,
-        map: u8,
-        key: ConstantIndex,
+        value: u8,
+        key: u8,
     },
     TryStart {
         arg_register: u8,
@@ -376,6 +381,16 @@ pub enum Instruction {
         register: u8,
         size: usize,
     },
+    StringStart {
+        register: u8,
+    },
+    StringPush {
+        register: u8,
+        value: u8,
+    },
+    StringFinish {
+        register: u8,
+    },
 }
 
 impl fmt::Display for Instruction {
@@ -393,12 +408,18 @@ impl fmt::Display for Instruction {
             LoadNonLocal { .. } => write!(f, "LoadNonLocal"),
             ValueExport { .. } => write!(f, "ValueExport"),
             Import { .. } => write!(f, "Import"),
-            MakeTuple { .. } => write!(f, "MakeTuple"),
             MakeTempTuple { .. } => write!(f, "MakeTempTuple"),
-            MakeList { .. } => write!(f, "MakeList"),
             MakeMap { .. } => write!(f, "MakeMap"),
             MakeNum2 { .. } => write!(f, "MakeNum2"),
             MakeNum4 { .. } => write!(f, "MakeNum4"),
+            SequenceStart { .. } => write!(f, "SequenceStart"),
+            SequencePush { .. } => write!(f, "SequencePush"),
+            SequencePushN { .. } => write!(f, "SequencePushN"),
+            SequenceToList { .. } => write!(f, "SequenceToList"),
+            SequenceToTuple { .. } => write!(f, "SequenceToTuple"),
+            StringStart { .. } => write!(f, "StringStart"),
+            StringPush { .. } => write!(f, "StringPush"),
+            StringFinish { .. } => write!(f, "StringFinish"),
             Range { .. } => write!(f, "Range"),
             RangeInclusive { .. } => write!(f, "RangeInclusive"),
             RangeTo { .. } => write!(f, "RangeTo"),
@@ -406,6 +427,7 @@ impl fmt::Display for Instruction {
             RangeFrom { .. } => write!(f, "RangeFrom"),
             RangeFull { .. } => write!(f, "RangeFull"),
             MakeIterator { .. } => write!(f, "MakeIterator"),
+            SimpleFunction { .. } => write!(f, "SimpleFunction"),
             Function { .. } => write!(f, "Function"),
             Capture { .. } => write!(f, "Capture"),
             Negate { .. } => write!(f, "Negate"),
@@ -438,10 +460,8 @@ impl fmt::Display for Instruction {
             SliceTo { .. } => write!(f, "SliceTo"),
             IsTuple { .. } => write!(f, "IsTuple"),
             IsList { .. } => write!(f, "IsList"),
-            ListPushValue { .. } => write!(f, "ListPushValue"),
-            ListPushValues { .. } => write!(f, "ListPushValues"),
-            ListUpdate { .. } => write!(f, "ListUpdate"),
             Index { .. } => write!(f, "Index"),
+            SetIndex { .. } => write!(f, "SetIndex"),
             MapInsert { .. } => write!(f, "MapInsert"),
             MetaInsert { .. } => write!(f, "MetaInsert"),
             MetaInsertNamed { .. } => write!(f, "MetaInsertNamed"),
@@ -489,18 +509,7 @@ impl fmt::Debug for Instruction {
             ValueExport { name, value } => {
                 write!(f, "ValueExport\tname: {}\tvalue: {}", name, value)
             }
-            Import { register, constant } => {
-                write!(f, "Import\t\tresult: {}\tconstant: {}", register, constant)
-            }
-            MakeTuple {
-                register,
-                start,
-                count,
-            } => write!(
-                f,
-                "MakeTuple\tresult: {}\tstart: {}\tcount: {}",
-                register, start, count
-            ),
+            Import { register } => write!(f, "Import\t\tregister: {}", register),
             MakeTempTuple {
                 register,
                 start,
@@ -510,14 +519,6 @@ impl fmt::Debug for Instruction {
                 "MakeTempTuple\tresult: {}\tstart: {}\tcount: {}",
                 register, start, count
             ),
-            MakeList {
-                register,
-                size_hint,
-            } => write!(
-                f,
-                "MakeList\tresult: {}\tsize_hint: {}",
-                register, size_hint
-            ),
             MakeMap {
                 register,
                 size_hint,
@@ -544,6 +545,28 @@ impl fmt::Debug for Instruction {
                 "MakeNum4\tresult: {}\tcount: {}\telement reg: {}",
                 register, count, element_register
             ),
+            SequenceStart {
+                register,
+                size_hint,
+            } => write!(
+                f,
+                "SequenceStart\tresult: {}\tsize_hint: {}",
+                register, size_hint
+            ),
+            SequencePush { sequence, value } => {
+                write!(f, "SequencePush\tsequence: {}\tvalue: {}", sequence, value)
+            }
+            SequencePushN {
+                sequence,
+                start,
+                count,
+            } => write!(
+                f,
+                "SequencePushN\tsequence: {}\tstart: {}\tcount: {}",
+                sequence, start, count
+            ),
+            SequenceToList { sequence } => write!(f, "SequenceToList\tsequence: {}", sequence),
+            SequenceToTuple { sequence } => write!(f, "SequenceToTuple\tsequence: {}", sequence),
             Range {
                 register,
                 start,
@@ -575,6 +598,15 @@ impl fmt::Debug for Instruction {
                 "MakeIterator\tresult: {}\titerable: {}",
                 register, iterable
             ),
+            SimpleFunction {
+                register,
+                arg_count,
+                size,
+            } => write!(
+                f,
+                "SimpleFunction\tresult: {}\targs: {}\t\tsize: {}",
+                register, arg_count, size,
+            ),
             Function {
                 register,
                 arg_count,
@@ -760,23 +792,6 @@ impl fmt::Debug for Instruction {
             IsList { register, value } => {
                 write!(f, "IsList\t\tresult: {}\tvalue: {}", register, value)
             }
-            ListPushValue { list, value } => {
-                write!(f, "ListPushValue\tlist: {}\tvalue: {}", list, value)
-            }
-            ListPushValues {
-                list,
-                values_start,
-                count,
-            } => write!(
-                f,
-                "ListPushValues\tlist: {}\t\tstart: {}\tcount: {}",
-                list, values_start, count
-            ),
-            ListUpdate { list, index, value } => write!(
-                f,
-                "ListUpdate\tlist: {}\t\tindex: {}\tvalue: {}",
-                list, index, value
-            ),
             Index {
                 register,
                 value,
@@ -786,6 +801,15 @@ impl fmt::Debug for Instruction {
                 "Index\t\tresult: {}\tvalue: {}\tindex: {}",
                 register, value, index
             ),
+            SetIndex {
+                register,
+                index,
+                value,
+            } => write!(
+                f,
+                "SetIndex\tregister: {}\t\tindex: {}\tvalue: {}",
+                register, index, value
+            ),
             MapInsert {
                 register,
                 value,
@@ -801,29 +825,33 @@ impl fmt::Debug for Instruction {
                 id,
             } => write!(
                 f,
-                "MetaInsert\tmap: {}\t\tvalue: {}\tid: {:?}",
-                register, value, id
+                "MetaInsert\tmap: {}\t\tid: {:?}\tvalue: {}",
+                register, id, value
             ),
             MetaInsertNamed {
                 register,
-                value,
                 id,
                 name,
+                value,
             } => write!(
                 f,
-                "MetaInsertNamed\tmap: {}\t\tvalue: {}\tid: {:?}\tname: {}",
-                register, value, id, name
+                "MetaInsertNamed\tmap: {}\t\tid: {:?}\tname: {}\t\tvalue: {}",
+                register, id, name, value
             ),
-            MetaExport { value, id } => write!(f, "MetaExport\tvalue: {}\tid: {:?}", value, id),
-            MetaExportNamed { value, id, name } => write!(
+            MetaExport { id, value } => write!(f, "MetaExport\tid: {:?}\tvalue: {}", id, value),
+            MetaExportNamed { id, name, value } => write!(
                 f,
-                "MetaExportNamed\tvalue: {}\tid: {:?}\tname: {}",
-                value, id, name
+                "MetaExportNamed\tid: {:?}\tname: {}\tvalue: {}",
+                id, name, value,
             ),
-            Access { register, map, key } => write!(
+            Access {
+                register,
+                value,
+                key,
+            } => write!(
                 f,
-                "Access\t\tresult: {}\tmap: {}\t\tkey: {}",
-                register, map, key
+                "Access\t\tresult: {}\tvalue: {}\tkey: {}",
+                register, value, key
             ),
             TryStart {
                 arg_register,
@@ -843,6 +871,15 @@ impl fmt::Debug for Instruction {
             CheckSize { register, size } => {
                 write!(f, "CheckSize\tregister: {}\tsize: {}", register, size)
             }
+            StringStart { register } => {
+                write!(f, "StringStart\tregister: {}", register)
+            }
+            StringPush { register, value } => {
+                write!(f, "StringPush\tregister: {}\tvalue: {}", register, value)
+            }
+            StringFinish { register } => {
+                write!(f, "StringFinish\tregister: {}", register)
+            }
         }
     }
 }
@@ -866,7 +903,7 @@ impl Iterator for InstructionReader {
     fn next(&mut self) -> Option<Self::Item> {
         use Instruction::*;
 
-        macro_rules! get_byte {
+        macro_rules! get_u8 {
             () => {{
                 match self.chunk.bytes.get(self.ip) {
                     Some(byte) => {
@@ -924,150 +961,178 @@ impl Iterator for InstructionReader {
 
         match op {
             Op::Copy => Some(Copy {
-                target: get_byte!(),
-                source: get_byte!(),
+                target: get_u8!(),
+                source: get_u8!(),
             }),
             Op::SetEmpty => Some(SetEmpty {
-                register: get_byte!(),
+                register: get_u8!(),
             }),
             Op::SetFalse => Some(SetBool {
-                register: get_byte!(),
+                register: get_u8!(),
                 value: false,
             }),
             Op::SetTrue => Some(SetBool {
-                register: get_byte!(),
+                register: get_u8!(),
                 value: true,
             }),
             Op::Set0 => Some(SetNumber {
-                register: get_byte!(),
+                register: get_u8!(),
                 value: 0,
             }),
             Op::Set1 => Some(SetNumber {
-                register: get_byte!(),
+                register: get_u8!(),
                 value: 1,
             }),
             Op::SetNumberU8 => Some(SetNumber {
-                register: get_byte!(),
-                value: get_byte!() as i64,
+                register: get_u8!(),
+                value: get_u8!() as i64,
             }),
             Op::LoadFloat => Some(LoadFloat {
-                register: get_byte!(),
-                constant: get_byte!() as ConstantIndex,
+                register: get_u8!(),
+                constant: ConstantIndex(get_u8!(), 0, 0),
             }),
-            Op::LoadFloatLong => Some(LoadFloat {
-                register: get_byte!(),
-                constant: get_u32!() as ConstantIndex,
+            Op::LoadFloat16 => Some(LoadFloat {
+                register: get_u8!(),
+                constant: ConstantIndex(get_u8!(), get_u8!(), 0),
+            }),
+            Op::LoadFloat24 => Some(LoadFloat {
+                register: get_u8!(),
+                constant: ConstantIndex(get_u8!(), get_u8!(), get_u8!()),
             }),
             Op::LoadInt => Some(LoadInt {
-                register: get_byte!(),
-                constant: get_byte!() as ConstantIndex,
+                register: get_u8!(),
+                constant: ConstantIndex(get_u8!(), 0, 0),
             }),
-            Op::LoadIntLong => Some(LoadInt {
-                register: get_byte!(),
-                constant: get_u32!() as ConstantIndex,
+            Op::LoadInt16 => Some(LoadInt {
+                register: get_u8!(),
+                constant: ConstantIndex(get_u8!(), get_u8!(), 0),
+            }),
+            Op::LoadInt24 => Some(LoadInt {
+                register: get_u8!(),
+                constant: ConstantIndex(get_u8!(), get_u8!(), get_u8!()),
             }),
             Op::LoadString => Some(LoadString {
-                register: get_byte!(),
-                constant: get_byte!() as ConstantIndex,
+                register: get_u8!(),
+                constant: ConstantIndex(get_u8!(), 0, 0),
             }),
-            Op::LoadStringLong => Some(LoadString {
-                register: get_byte!(),
-                constant: get_u32!() as ConstantIndex,
+            Op::LoadString16 => Some(LoadString {
+                register: get_u8!(),
+                constant: ConstantIndex(get_u8!(), get_u8!(), 0),
+            }),
+            Op::LoadString24 => Some(LoadString {
+                register: get_u8!(),
+                constant: ConstantIndex(get_u8!(), get_u8!(), get_u8!()),
             }),
             Op::LoadNonLocal => Some(LoadNonLocal {
-                register: get_byte!(),
-                constant: get_byte!() as ConstantIndex,
+                register: get_u8!(),
+                constant: ConstantIndex(get_u8!(), 0, 0),
             }),
-            Op::LoadNonLocalLong => Some(LoadNonLocal {
-                register: get_byte!(),
-                constant: get_u32!() as ConstantIndex,
+            Op::LoadNonLocal16 => Some(LoadNonLocal {
+                register: get_u8!(),
+                constant: ConstantIndex(get_u8!(), get_u8!(), 0),
+            }),
+            Op::LoadNonLocal24 => Some(LoadNonLocal {
+                register: get_u8!(),
+                constant: ConstantIndex(get_u8!(), get_u8!(), get_u8!()),
             }),
             Op::ValueExport => Some(ValueExport {
-                name: get_byte!() as ConstantIndex,
-                value: get_byte!(),
-            }),
-            Op::ValueExportLong => Some(ValueExport {
-                name: get_u32!() as ConstantIndex,
-                value: get_byte!(),
+                name: get_u8!(),
+                value: get_u8!(),
             }),
             Op::Import => Some(Import {
-                register: get_byte!(),
-                constant: get_byte!() as ConstantIndex,
-            }),
-            Op::ImportLong => Some(Import {
-                register: get_byte!(),
-                constant: get_u32!() as ConstantIndex,
-            }),
-            Op::MakeTuple => Some(MakeTuple {
-                register: get_byte!(),
-                start: get_byte!(),
-                count: get_byte!(),
+                register: get_u8!(),
             }),
             Op::MakeTempTuple => Some(MakeTempTuple {
-                register: get_byte!(),
-                start: get_byte!(),
-                count: get_byte!(),
-            }),
-            Op::MakeList => Some(MakeList {
-                register: get_byte!(),
-                size_hint: get_byte!() as usize,
-            }),
-            Op::MakeListLong => Some(MakeList {
-                register: get_byte!(),
-                size_hint: get_u32!() as usize,
+                register: get_u8!(),
+                start: get_u8!(),
+                count: get_u8!(),
             }),
             Op::MakeMap => Some(MakeMap {
-                register: get_byte!(),
-                size_hint: get_byte!() as usize,
+                register: get_u8!(),
+                size_hint: get_u8!() as usize,
             }),
-            Op::MakeMapLong => Some(MakeMap {
-                register: get_byte!(),
+            Op::MakeMap32 => Some(MakeMap {
+                register: get_u8!(),
                 size_hint: get_u32!() as usize,
             }),
             Op::MakeNum2 => Some(MakeNum2 {
-                register: get_byte!(),
-                count: get_byte!(),
-                element_register: get_byte!(),
+                register: get_u8!(),
+                count: get_u8!(),
+                element_register: get_u8!(),
             }),
             Op::MakeNum4 => Some(MakeNum4 {
-                register: get_byte!(),
-                count: get_byte!(),
-                element_register: get_byte!(),
+                register: get_u8!(),
+                count: get_u8!(),
+                element_register: get_u8!(),
+            }),
+            Op::SequenceStart => Some(SequenceStart {
+                register: get_u8!(),
+                size_hint: get_u8!() as usize,
+            }),
+            Op::SequenceStart32 => Some(SequenceStart {
+                register: get_u8!(),
+                size_hint: get_u32!() as usize,
+            }),
+            Op::SequencePush => Some(SequencePush {
+                sequence: get_u8!(),
+                value: get_u8!(),
+            }),
+            Op::SequencePushN => Some(SequencePushN {
+                sequence: get_u8!(),
+                start: get_u8!(),
+                count: get_u8!(),
+            }),
+            Op::SequenceToList => Some(SequenceToList {
+                sequence: get_u8!(),
+            }),
+            Op::SequenceToTuple => Some(SequenceToTuple {
+                sequence: get_u8!(),
             }),
             Op::Range => Some(Range {
-                register: get_byte!(),
-                start: get_byte!(),
-                end: get_byte!(),
+                register: get_u8!(),
+                start: get_u8!(),
+                end: get_u8!(),
             }),
             Op::RangeInclusive => Some(RangeInclusive {
-                register: get_byte!(),
-                start: get_byte!(),
-                end: get_byte!(),
+                register: get_u8!(),
+                start: get_u8!(),
+                end: get_u8!(),
             }),
             Op::RangeTo => Some(RangeTo {
-                register: get_byte!(),
-                end: get_byte!(),
+                register: get_u8!(),
+                end: get_u8!(),
             }),
             Op::RangeToInclusive => Some(RangeToInclusive {
-                register: get_byte!(),
-                end: get_byte!(),
+                register: get_u8!(),
+                end: get_u8!(),
             }),
             Op::RangeFrom => Some(RangeFrom {
-                register: get_byte!(),
-                start: get_byte!(),
+                register: get_u8!(),
+                start: get_u8!(),
             }),
             Op::RangeFull => Some(RangeFull {
-                register: get_byte!(),
+                register: get_u8!(),
             }),
             Op::MakeIterator => Some(MakeIterator {
-                register: get_byte!(),
-                iterable: get_byte!(),
+                register: get_u8!(),
+                iterable: get_u8!(),
             }),
+            Op::SimpleFunction => {
+                let register = get_u8!();
+                let arg_count = get_u8!();
+                let size = get_u16!() as usize;
+
+                Some(SimpleFunction {
+                    register,
+                    arg_count,
+                    size,
+                })
+            }
             Op::Function => {
-                let register = get_byte!();
-                let arg_count = get_byte!();
-                let capture_count = get_byte!();
-                let flags = FunctionFlags::from_byte(get_byte!());
+                let register = get_u8!();
+                let arg_count = get_u8!();
+                let capture_count = get_u8!();
+                let flags = FunctionFlags::from_byte(get_u8!());
                 let size = get_u16!() as usize;
 
                 Some(Function {
@@ -1081,79 +1146,79 @@ impl Iterator for InstructionReader {
                 })
             }
             Op::Capture => Some(Capture {
-                function: get_byte!(),
-                target: get_byte!(),
-                source: get_byte!(),
+                function: get_u8!(),
+                target: get_u8!(),
+                source: get_u8!(),
             }),
             Op::Negate => Some(Negate {
-                register: get_byte!(),
-                source: get_byte!(),
+                register: get_u8!(),
+                source: get_u8!(),
             }),
             Op::Add => Some(Add {
-                register: get_byte!(),
-                lhs: get_byte!(),
-                rhs: get_byte!(),
+                register: get_u8!(),
+                lhs: get_u8!(),
+                rhs: get_u8!(),
             }),
             Op::Subtract => Some(Subtract {
-                register: get_byte!(),
-                lhs: get_byte!(),
-                rhs: get_byte!(),
+                register: get_u8!(),
+                lhs: get_u8!(),
+                rhs: get_u8!(),
             }),
             Op::Multiply => Some(Multiply {
-                register: get_byte!(),
-                lhs: get_byte!(),
-                rhs: get_byte!(),
+                register: get_u8!(),
+                lhs: get_u8!(),
+                rhs: get_u8!(),
             }),
             Op::Divide => Some(Divide {
-                register: get_byte!(),
-                lhs: get_byte!(),
-                rhs: get_byte!(),
+                register: get_u8!(),
+                lhs: get_u8!(),
+                rhs: get_u8!(),
             }),
             Op::Modulo => Some(Modulo {
-                register: get_byte!(),
-                lhs: get_byte!(),
-                rhs: get_byte!(),
+                register: get_u8!(),
+                lhs: get_u8!(),
+                rhs: get_u8!(),
             }),
             Op::Less => Some(Less {
-                register: get_byte!(),
-                lhs: get_byte!(),
-                rhs: get_byte!(),
+                register: get_u8!(),
+                lhs: get_u8!(),
+                rhs: get_u8!(),
             }),
             Op::LessOrEqual => Some(LessOrEqual {
-                register: get_byte!(),
-                lhs: get_byte!(),
-                rhs: get_byte!(),
+                register: get_u8!(),
+                lhs: get_u8!(),
+                rhs: get_u8!(),
             }),
             Op::Greater => Some(Greater {
-                register: get_byte!(),
-                lhs: get_byte!(),
-                rhs: get_byte!(),
+                register: get_u8!(),
+                lhs: get_u8!(),
+                rhs: get_u8!(),
             }),
             Op::GreaterOrEqual => Some(GreaterOrEqual {
-                register: get_byte!(),
-                lhs: get_byte!(),
-                rhs: get_byte!(),
+                register: get_u8!(),
+                lhs: get_u8!(),
+                rhs: get_u8!(),
             }),
             Op::Equal => Some(Equal {
-                register: get_byte!(),
-                lhs: get_byte!(),
-                rhs: get_byte!(),
+                register: get_u8!(),
+                lhs: get_u8!(),
+                rhs: get_u8!(),
             }),
             Op::NotEqual => Some(NotEqual {
-                register: get_byte!(),
-                lhs: get_byte!(),
-                rhs: get_byte!(),
+                register: get_u8!(),
+                lhs: get_u8!(),
+                rhs: get_u8!(),
             }),
             Op::Jump => Some(Jump {
                 offset: get_u16!() as usize,
             }),
             Op::JumpTrue => Some(JumpIf {
-                register: get_byte!(),
+                register: get_u8!(),
                 offset: get_u16!() as usize,
                 jump_condition: true,
             }),
             Op::JumpFalse => Some(JumpIf {
-                register: get_byte!(),
+                register: get_u8!(),
                 offset: get_u16!() as usize,
                 jump_condition: false,
             }),
@@ -1161,106 +1226,92 @@ impl Iterator for InstructionReader {
                 offset: get_u16!() as usize,
             }),
             Op::JumpBackFalse => Some(JumpBackIf {
-                register: get_byte!(),
+                register: get_u8!(),
                 offset: get_u16!() as usize,
                 jump_condition: false,
             }),
             Op::Call => Some(Call {
-                result: get_byte!(),
-                function: get_byte!(),
-                frame_base: get_byte!(),
-                arg_count: get_byte!(),
+                result: get_u8!(),
+                function: get_u8!(),
+                frame_base: get_u8!(),
+                arg_count: get_u8!(),
             }),
             Op::CallChild => Some(CallChild {
-                result: get_byte!(),
-                function: get_byte!(),
-                frame_base: get_byte!(),
-                arg_count: get_byte!(),
-                parent: get_byte!(),
+                result: get_u8!(),
+                function: get_u8!(),
+                frame_base: get_u8!(),
+                arg_count: get_u8!(),
+                parent: get_u8!(),
             }),
             Op::Return => Some(Return {
-                register: get_byte!(),
+                register: get_u8!(),
             }),
             Op::Yield => Some(Yield {
-                register: get_byte!(),
+                register: get_u8!(),
             }),
             Op::Throw => Some(Throw {
-                register: get_byte!(),
+                register: get_u8!(),
             }),
             Op::Size => Some(Size {
-                register: get_byte!(),
-                value: get_byte!(),
+                register: get_u8!(),
+                value: get_u8!(),
             }),
             Op::IterNext => Some(IterNext {
-                register: get_byte!(),
-                iterator: get_byte!(),
+                register: get_u8!(),
+                iterator: get_u8!(),
                 jump_offset: get_u16!() as usize,
             }),
             Op::IterNextTemp => Some(IterNextTemp {
-                register: get_byte!(),
-                iterator: get_byte!(),
+                register: get_u8!(),
+                iterator: get_u8!(),
                 jump_offset: get_u16!() as usize,
             }),
             Op::IterNextQuiet => Some(IterNextQuiet {
-                iterator: get_byte!(),
+                iterator: get_u8!(),
                 jump_offset: get_u16!() as usize,
             }),
             Op::ValueIndex => Some(ValueIndex {
-                register: get_byte!(),
-                value: get_byte!(),
-                index: get_byte!() as i8,
+                register: get_u8!(),
+                value: get_u8!(),
+                index: get_u8!() as i8,
             }),
             Op::SliceFrom => Some(SliceFrom {
-                register: get_byte!(),
-                value: get_byte!(),
-                index: get_byte!() as i8,
+                register: get_u8!(),
+                value: get_u8!(),
+                index: get_u8!() as i8,
             }),
             Op::SliceTo => Some(SliceTo {
-                register: get_byte!(),
-                value: get_byte!(),
-                index: get_byte!() as i8,
+                register: get_u8!(),
+                value: get_u8!(),
+                index: get_u8!() as i8,
             }),
             Op::IsTuple => Some(IsTuple {
-                register: get_byte!(),
-                value: get_byte!(),
+                register: get_u8!(),
+                value: get_u8!(),
             }),
             Op::IsList => Some(IsList {
-                register: get_byte!(),
-                value: get_byte!(),
-            }),
-            Op::ListPushValue => Some(ListPushValue {
-                list: get_byte!(),
-                value: get_byte!(),
-            }),
-            Op::ListPushValues => Some(ListPushValues {
-                list: get_byte!(),
-                values_start: get_byte!(),
-                count: get_byte!(),
-            }),
-            Op::ListUpdate => Some(ListUpdate {
-                list: get_byte!(),
-                index: get_byte!(),
-                value: get_byte!(),
+                register: get_u8!(),
+                value: get_u8!(),
             }),
             Op::Index => Some(Index {
-                register: get_byte!(),
-                value: get_byte!(),
-                index: get_byte!(),
+                register: get_u8!(),
+                value: get_u8!(),
+                index: get_u8!(),
+            }),
+            Op::SetIndex => Some(SetIndex {
+                register: get_u8!(),
+                index: get_u8!(),
+                value: get_u8!(),
             }),
             Op::MapInsert => Some(MapInsert {
-                register: get_byte!(),
-                value: get_byte!(),
-                key: get_byte!() as ConstantIndex,
-            }),
-            Op::MapInsertLong => Some(MapInsert {
-                register: get_byte!(),
-                value: get_byte!(),
-                key: get_u32!() as ConstantIndex,
+                register: get_u8!(),
+                key: get_u8!(),
+                value: get_u8!(),
             }),
             Op::MetaInsert => {
-                let register = get_byte!();
-                let value = get_byte!();
-                let meta_id = get_byte!();
+                let register = get_u8!();
+                let meta_id = get_u8!();
+                let value = get_u8!();
                 if let Ok(id) = meta_id.try_into() {
                     Some(MetaInsert {
                         register,
@@ -1277,31 +1328,10 @@ impl Iterator for InstructionReader {
                 }
             }
             Op::MetaInsertNamed => {
-                let register = get_byte!();
-                let value = get_byte!();
-                let meta_id = get_byte!();
-                let name = get_byte!() as ConstantIndex;
-                if let Ok(id) = meta_id.try_into() {
-                    Some(MetaInsertNamed {
-                        register,
-                        value,
-                        id,
-                        name,
-                    })
-                } else {
-                    Some(Error {
-                        message: format!(
-                            "Unexpected meta id {} found at instruction {}",
-                            meta_id, op_ip
-                        ),
-                    })
-                }
-            }
-            Op::MetaInsertNamedLong => {
-                let register = get_byte!();
-                let value = get_byte!();
-                let meta_id = get_byte!();
-                let name = get_u32!() as ConstantIndex;
+                let register = get_u8!();
+                let meta_id = get_u8!();
+                let name = get_u8!();
+                let value = get_u8!();
                 if let Ok(id) = meta_id.try_into() {
                     Some(MetaInsertNamed {
                         register,
@@ -1319,8 +1349,8 @@ impl Iterator for InstructionReader {
                 }
             }
             Op::MetaExport => {
-                let meta_id = get_byte!();
-                let value = get_byte!();
+                let meta_id = get_u8!();
+                let value = get_u8!();
                 if let Ok(id) = meta_id.try_into() {
                     Some(MetaExport { id, value })
                 } else {
@@ -1333,24 +1363,9 @@ impl Iterator for InstructionReader {
                 }
             }
             Op::MetaExportNamed => {
-                let meta_id = get_byte!();
-                let value = get_byte!();
-                let name = get_byte!() as ConstantIndex;
-                if let Ok(id) = meta_id.try_into() {
-                    Some(MetaExportNamed { id, value, name })
-                } else {
-                    Some(Error {
-                        message: format!(
-                            "Unexpected meta id {} found at instruction {}",
-                            meta_id, op_ip
-                        ),
-                    })
-                }
-            }
-            Op::MetaExportNamedLong => {
-                let meta_id = get_byte!();
-                let value = get_byte!();
-                let name = get_u32!() as ConstantIndex;
+                let meta_id = get_u8!();
+                let name = get_u8!();
+                let value = get_u8!();
                 if let Ok(id) = meta_id.try_into() {
                     Some(MetaExportNamed { id, value, name })
                 } else {
@@ -1363,27 +1378,22 @@ impl Iterator for InstructionReader {
                 }
             }
             Op::Access => Some(Access {
-                register: get_byte!(),
-                map: get_byte!(),
-                key: get_byte!() as ConstantIndex,
-            }),
-            Op::AccessLong => Some(Access {
-                register: get_byte!(),
-                map: get_byte!(),
-                key: get_u32!() as ConstantIndex,
+                register: get_u8!(),
+                value: get_u8!(),
+                key: get_u8!(),
             }),
             Op::TryStart => Some(TryStart {
-                arg_register: get_byte!(),
+                arg_register: get_u8!(),
                 catch_offset: get_u16!() as usize,
             }),
             Op::TryEnd => Some(TryEnd),
             Op::Debug => Some(Debug {
-                register: get_byte!(),
-                constant: get_u32!() as ConstantIndex,
+                register: get_u8!(),
+                constant: ConstantIndex(get_u8!(), get_u8!(), get_u8!()),
             }),
             Op::CheckType => {
-                let register = get_byte!();
-                match TypeId::from_byte(get_byte!()) {
+                let register = get_u8!();
+                match TypeId::from_byte(get_u8!()) {
                     Ok(type_id) => Some(CheckType { register, type_id }),
                     Err(byte) => Some(Error {
                         message: format!("Unexpected value for CheckType id: {}", byte),
@@ -1391,8 +1401,18 @@ impl Iterator for InstructionReader {
                 }
             }
             Op::CheckSize => Some(CheckSize {
-                register: get_byte!(),
-                size: get_byte!() as usize,
+                register: get_u8!(),
+                size: get_u8!() as usize,
+            }),
+            Op::StringStart => Some(StringStart {
+                register: get_u8!(),
+            }),
+            Op::StringPush => Some(StringPush {
+                register: get_u8!(),
+                value: get_u8!(),
+            }),
+            Op::StringFinish => Some(StringFinish {
+                register: get_u8!(),
             }),
             _ => Some(Error {
                 message: format!("Unexpected opcode {:?} found at instruction {}", op, op_ip),
